/*
 * A flexpage abstraction.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mem/memory_utils.h>
#include <mem/region.h>
#include <mem/send_flexpage.h>



/* Work around API change. */

#ifndef L4RE_DS_MAP_FLAG_RO
#define L4RE_DS_MAP_FLAG_RO L4RE_DS_F_R
#endif

#ifndef L4RE_DS_MAP_FLAG_RW
#define L4RE_DS_MAP_FLAG_RW L4RE_DS_F_RW
#endif



/* A flexpage abstraction. */

class Flexpage
{
protected:
    unsigned int _counter;
    map_flags_t _flags;

public:
    Region *region;

    /* General flexpage characteristics. */

    offset_t base_addr, size;
    offset_t base_offset;

    /* Transient debugging information. */

    offset_t page_addr, page_offset;

    /* Associate a flexpage with a memory 'region'. */

    explicit Flexpage(Region *region) : region(region)
    {
    }

    void reset(offset_t offset);

    bool decrement();

    void increment();

    void invalidate();

    bool valid();

    bool supports_position(offset_t position);

    void upgrade(map_flags_t flags);

    bool modified();

    SendFlexpage to_send(offset_t offset, offset_t hot_spot, map_flags_t flags,
                         offset_t max_offset=0);

    SendFlexpage to_unmap();
};

// vim: tabstop=4 expandtab shiftwidth=4
